@@ -78,25 +78,38 @@ def new_kitchen():
 
 
 def nodes_with_role(rolename):
-    """Sets a list of nodes that have the given role
-    in their run list and calls node()
-
-    """
-    nodes_in_env = []
-    nodes = lib.get_nodes_with_role(rolename)
-    if env.chef_environment is None:
-        # Pass all nodes
-        nodes_in_env = [n['name'] for n in nodes]
-    else:
-        # Only nodes in environment
-        nodes_in_env = [n['name'] for n in nodes \
-                        if n.get('chef_environment') == env.chef_environment]
-    if not len(nodes_in_env):
+    """Configures a list of nodes that have the given role in their run list"""
+    nodes = [n['name'] for n in
+             lib.get_nodes_with_role(rolename, env.chef_environment)]
+    if not len(nodes):
         print("No nodes found with role '{0}'".format(rolename))
         sys.exit(0)
-    return node(*nodes_in_env)
+    return node(*nodes)
 
 
+def nodes_with_recipe(recipename):
+    """Configures a list of nodes that have the given recipe in their run list
+    """
+    nodes = [n['name'] for n in
+             lib.get_nodes_with_recipe(recipename, env.chef_environment)]
+    if not len(nodes):
+        print("No nodes found with recipe '{0}'".format(recipename))
+        sys.exit(0)
+    return node(*nodes)
+
+
+def nodes_with_tag(tag):
+    """Sets a list of nodes that have the given tag assigned and calls node()"""
+    nodes = lib.get_nodes_with_tag(tag, env.chef_environment,
+                                   littlechef.include_guests)
+    nodes = [n['name'] for n in nodes]
+    if not len(nodes):
+        print("No nodes found with tag '{0}'".format(tag))
+        sys.exit(0)
+    return node(*nodes)
+
+
+@hosts('setup')
 def node(*nodes):
     """Selects and configures a list of nodes. 'all' configures all nodes"""
     chef.build_node_data_bag()
@@ -124,7 +137,9 @@ def node(*nodes):
     # Check whether another command was given in addition to "node:"
     if not(littlechef.__cooking__ and
             'node:' not in sys.argv[-1] and
-            'nodes_with_role:' not in sys.argv[-1]):
+            'nodes_with_role:' not in sys.argv[-1] and
+            'nodes_with_recipe:' not in sys.argv[-1] and
+            'nodes_with_tag:' not in sys.argv[-1]):
         # If user didn't type recipe:X, role:Y or deploy_chef,
         # configure the nodes
         with settings():
@@ -140,11 +155,7 @@ def _configure_fabric_for_platform(platform):
 
 def _node_runner():
     """This is only used by node so that we can execute in parallel"""
-    if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODES recipe:MYRECIPE')
-    if '@' in env.host_string:
-        env.user = env.host_string.split('@')[0]
-    env.host_string = lib.resolve_hostname(env.host_string)
+    env.host_string = lib.get_env_host_string()
     node = lib.get_node(env.host_string)
 
     _configure_fabric_for_platform(node.get("platform"))
@@ -159,8 +170,7 @@ def _node_runner():
 def deploy_chef(gems="no", ask="yes", version="0.10", distro_type=None,
                 distro=None, platform=None, stop_client='yes', method=None):
     """Install chef-solo on a node"""
-    if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODES deploy_chef')
+    env.host_string = lib.get_env_host_string()
     deprecated_parameters = [distro_type, distro, platform]
     if any(param is not None for param in deprecated_parameters) or gems != 'no':
         print("DeprecationWarning: the parameters 'gems', distro_type',"
@@ -206,7 +216,7 @@ def deploy_chef(gems="no", ask="yes", version="0.10", distro_type=None,
                       ":\n  {0}".format(output))
             node = {"run_list": []}
             for attribute in ["ipaddress", "platform", "platform_family",
-                         "platform_version"]:
+                              "platform_version"]:
                 if ohai.get(attribute):
                     node[attribute] = ohai[attribute]
             chef.save_config(node)
@@ -218,13 +228,11 @@ def recipe(recipe):
     If no nodes/hostname.json file exists, it creates one
 
     """
-    # Check that a node has been selected
-    if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODES recipe:MYRECIPE')
+    env.host_string = lib.get_env_host_string()
     lib.print_header(
         "Applying recipe '{0}' on node {1}".format(recipe, env.host_string))
 
-    # Now create configuration and sync node
+    # Create configuration and sync node
     data = lib.get_node(env.host_string)
     data["run_list"] = ["recipe[{0}]".format(recipe)]
     if not __testing__:
@@ -237,9 +245,7 @@ def role(role):
     If no nodes/hostname.json file exists, it creates one
 
     """
-    # Check that a node has been selected
-    if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODES role:MYROLE')
+    env.host_string = lib.get_env_host_string()
     lib.print_header(
         "Applying role '{0}' to {1}".format(role, env.host_string))
 
@@ -252,8 +258,7 @@ def role(role):
 
 def ssh(name):
     """Executes the given command"""
-    if not env.host_string:
-        abort('no node specified\nUsage: fix node:MYNODES ssh:COMMAND')
+    env.host_string = lib.get_env_host_string()
     print("\nExecuting the command '{0}' on node {1}...".format(
           name, env.host_string))
     # Execute remotely using either the sudo or the run fabric functions
@@ -269,8 +274,7 @@ def plugin(name):
     Plugins are expected to be found in the kitchen's 'plugins' directory
 
     """
-    if not env.host_string:
-        abort('No node specified\nUsage: fix node:MYNODES plugin:MYPLUGIN')
+    env.host_string = lib.get_env_host_string()
     plug = lib.import_plugin(name)
     lib.print_header("Executing plugin '{0}' on "
                      "{1}".format(name, env.host_string))
@@ -295,13 +299,13 @@ def list_nodes_detailed():
 
 @hosts('api')
 def list_nodes_with_recipe(recipe):
-    """Show all nodes which have asigned a given recipe"""
+    """Show all nodes which have assigned a given recipe"""
     lib.print_nodes(lib.get_nodes_with_recipe(recipe, env.chef_environment))
 
 
 @hosts('api')
 def list_nodes_with_role(role):
-    """Show all nodes which have asigned a given role"""
+    """Show all nodes which have assigned a given role"""
     lib.print_nodes(lib.get_nodes_with_role(role, env.chef_environment))
 
 
@@ -315,6 +319,13 @@ def list_envs():
             env.get('description', '(no description)')))
 
 
+@hosts('api')
+def list_nodes_with_tag(tag):
+    """Show all nodes which have assigned a given tag"""
+    lib.print_nodes(lib.get_nodes_with_tag(tag, env.chef_environment,
+                                           littlechef.include_guests))
+
+
 @hosts('api')
 def list_recipes():
     """Show a list of all available recipes"""
@@ -471,8 +482,10 @@ def _readconfig():
     except ConfigParser.NoOptionError:
         pass
 
-    if user_specified and not env.password and not env.key_filename and not env.ssh_config:
-        abort('You need to define a password, keypair file, or ssh-config file in config.cfg')
+    if (user_specified and not env.password and not env.key_filename
+            and not env.ssh_config):
+        abort('You need to define a password, keypair file, or ssh-config '
+              'file in config.cfg')
 
     # Node's Chef Solo working directory for storing cookbooks, roles, etc.
     try:
@@ -490,6 +503,19 @@ def _readconfig():
     except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
         env.follow_symlinks = False
 
+    # Upload Directory
+    try:
+        env.sync_packages_dest_dir = config.get('sync-packages',
+                                                'dest-dir')
+    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+        env.sync_packages_dest_dir = None
+
+    # Local Directory
+    try:
+        env.sync_packages_local_dir = config.get('sync-packages',
+                                                 'local-dir')
+    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+        env.sync_packages_local_dir = None
 
 # Only read config if fix is being used and we are not creating a new kitchen
 if littlechef.__cooking__:
